#!/bin/bash

DATE=`date '+%Y-%m-%d %H:%M:%S'`
logFile=/data/log/catalog_restore_wrapper_.log
BASEDIR=$(dirname $0)

logMsg(){
	echo $1 >> ${logFile}
}

failed_exit(){		
	logMsg "++-------------------------------------------------------------++"
	exit 1
}


clean_exit(){		
	logMsg "++-------------------------------------------------------------++"
	exit 0
}


logMsg "DATE: ${DATE}"
logMsg "Arguments: $*"
logMsg ""

mode=$1

if [ ${mode} == "initialize" ]; then

	if [ $# -ne 2 ]; then
 	    logMsg "$0 incorrect number of arguments."
	    failed_exit
	fi

	if ! [[ $2 =~ ^/tmp/ ]] ; then
	    logMsg "Directory $2 is not allowed to use for restore."
	    failed_exit
	fi

        logMsg "Stopping virgo"
        systemctl stop virgo
        logMsg "Done stopping virgo"
        sleep 5
        chmod -R 777 $2
elif [ ${mode} == "finalize" ]; then

	if [ $# -ne 1 ]; then
 	    logMsg "$0 incorrect number of arguments."
	    failed_exit
	fi

        chown virgo:virgo /data3/lucene/lucidx/
        chown virgo:virgo /data3/lucene/lucidx/*
	if [ -f ${BASEDIR}/managedb_wrapper.sh ]; then
		LOG_DATE=`date +%s`
		LOG_PATH="/data/log/adminconsole/managedb-catalogrestore-replay-changelog-${LOG_DATE}.log"
		logMsg "Replaying change log. Logfile: ${LOG_PATH} Basedir: ${BASEDIR}"
		${BASEDIR}/managedb_wrapper.sh --replay-changelog --log ${LOG_PATH}
		if [ $? != 0]; then
			logMsg "Failure in replaying change log"
		else
			logMsg "Done replaying change log"
		fi
	else
		logMsg "Not replaying changelog. ${BASEDIR}/managedb_wrapper.sh NOT found"
	fi
        logMsg "Restore has been completed. Starting virgo"
        systemctl start virgo
        logMsg "Started Virgo"
fi

clean_exit
