#!/usr/bin/python
from __future__ import print_function
import os , sys, json , re
from subprocess import call ,Popen, PIPE

#-------------------------------------------------------------
# main function
#-------------------------------------------------------------
def main():
    endpoints_list = parse_regions_json()
    create_aws_cert(endpoints_list)
    update_keystore(endpoints_list)
    cleanup(endpoints_list)

#-------------------------------------------------------------
# generate_crt (region, cmd)
#   returns 0 if Succeed 
#-------------------------------------------------------------
def generate_crt (region, cmd):
    print ( 'Generating certificate for ' + region + ':443 ...',end="")

    if debug: print ( 'Debug: region:' + region + ' , command:' + cmd )

    p = Popen( cmd , shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate()
    
    list=stdout.splitlines()

    CERTIFICATE_FILE = CRT_LOCATION + '/'+ region + '.crt' 

    crt_file = open(CERTIFICATE_FILE,'w')
    key_seen=False
    for line in list:

        if line.startswith("-----BEGIN CERTIFICATE-----"):
            key_seen=True
            crt_file.write(line + '\n')
            continue
        if line.startswith("-----END CERTIFICATE-----"):
            key_seen=False
            crt_file.write(line + '\n')
            break
        if key_seen:
            crt_file.write(line + '\n')
    crt_file.close()

    if p.returncode: 
        print ("FAILED")
        print (stdout)
    else:
        print ("SUCCEED")

    return p.returncode

#-------------------------------------------------------------
# execute_cmd (command)
#   returns error code from cmd command
#-------------------------------------------------------------
def execute_cmd (cmd):
    p = Popen( cmd , shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate()

    if debug: print (cmd)
    if debug: 
        if p.returncode: print (stdout)

    return p.returncode

#-------------------------------------------------------------
# parse_regions_json()
#   returns list of endpoints
#-------------------------------------------------------------
def parse_regions_json():

    endpoint_list = []
    with open(REGIONS) as data_file: 
        data = json.load(data_file)
        
    for region in data["regions"]["region"]:
        #print ('Region: ' + region["systemname"] )
        for service in region["service"]:
            endpoint=service["endpoint"]
            endpoint_base=re.sub("^https://|/$", "", endpoint)
            endpoint_list.append ( endpoint_base ) # add 443 
            
    return endpoint_list

#-------------------------------------------------------------
# create_aws_cert( endpoints )
#   returns 0 if Succeed 
#-------------------------------------------------------------
def create_aws_cert(endpoints):
    for endpoint in endpoints:
        if debug: print (endpoint)
        cmd='openssl s_client -connect ' + endpoint +  ':443'
        generate_crt (endpoint , cmd ) 

#-------------------------------------------------------------
# check_alias_in_keystore( endpoints ). Delete certificate 
#  if alias exists. 
#   returns 0 if Succeed 
#-------------------------------------------------------------
def check_alias_in_keystore(endpoint):
    cmd="/bin/keytool -list  -keystore  /opt/virgo/configuration/keystore -storepass ecx-beta -alias " + endpoint
    cmd_rc=execute_cmd(cmd)
    
    if cmd_rc:
        if debug: print (endpoint + ' does not exist, nothing to do.')
    else:
        print ( 'Alias ' + endpoint + ' exists in keystore, deleting ... ',end="")
        # delete alias
        cmd2="/bin/keytool  -delete -alias " + endpoint  +  " -keystore /opt/virgo/configuration/keystore  -storepass ecx-beta"
        cmd2_rc=execute_cmd(cmd2)

        if cmd2_rc: print("FAILED")
        else: print ("SUCCEED")


#-------------------------------------------------------------
# import_certificates( endpoint). Endpoint is same as alias 
#   returns 0 if Succeed 
#-------------------------------------------------------------
def import_certificate(endpoint):
    if debug: print (endpoint)
    certificate_file = CRT_LOCATION + '/' + endpoint + '.crt'
    if os.path.exists ( certificate_file):
        print ( 'Importing certificate ' +  certificate_file + ' ... ', end="")
        cmd = "/bin/keytool -import --noprompt -alias " + endpoint + " -keystore /opt/virgo/configuration/keystore -storepass ecx-beta -file " + certificate_file
        rc=execute_cmd(cmd)
        
        if rc: print("FAILED")
        else: print ("SUCCEED")


#-------------------------------------------------------------
# update_keystore(endpoints)
#-------------------------------------------------------------
def update_keystore(endpoints):
    for endpoint in endpoints:
        check_alias_in_keystore(endpoint)
        import_certificate(endpoint)
    

#-------------------------------------------------------------
# cleanup(). Deletes temporary certificates files
#-------------------------------------------------------------
def cleanup(endpoints):
  for endpoint in endpoints:
      certificate_file = CRT_LOCATION + '/' + endpoint + '.crt'
      if os.path.exists (certificate_file): os.remove(certificate_file)


if __name__ == "__main__":

    REGIONS="/opt/virgo/repository/ecx-usr/com.catalogic.ecx.cloud.aws.impl.regions.json"
    CRT_LOCATION='/tmp'
    debug=0

    main()

